package betterwithmods.common.blocks.tile;

import net.minecraft.entity.Entity;
import net.minecraft.entity.IProjectile;
import net.minecraft.entity.item.EntityXPOrb;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.ITickable;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;

import javax.annotation.Nullable;
import java.util.List;

/**
 * Purpose:
 *
 * @author Tyler Marshall
 * @version 3/20/17
 */
public class TileEntityDragonVessel extends TileEntity implements ITickable {
    private final int MAX_EXPERIENCE = 800;
    private int experience;
    private int maxDist = 5;
    public TileEntityDragonVessel() {
    }

    private int addExperience(int xp)  {
        if(this.experience >= MAX_EXPERIENCE)
            return xp;
        if(this.experience < MAX_EXPERIENCE-xp) {
            this.experience += xp;
            return 0;
        } else if( this.experience >= MAX_EXPERIENCE-xp) {
            int newXp = xp - (MAX_EXPERIENCE-experience);
            this.experience += newXp;
            return xp - newXp;
        }
        return xp;
    }
    private void hooverXP(EntityXPOrb entity) {
        if (!field_145850_b.field_72995_K && !entity.field_70128_L) {
            int xpValue = entity.func_70526_d();
            addExperience(xpValue);
            func_70296_d();
            if (xpValue > 0) {
                entity.field_70530_e = xpValue;
                release(entity);
            } else {
                entity.func_70106_y();
            }
        }
    }

    @Override
    public void func_73660_a() {
        AxisAlignedBB box = field_145850_b.func_180495_p(field_174879_c).func_185900_c(field_145850_b,field_174879_c).func_72321_a(5,5,5);
        List<EntityXPOrb> xp = field_145850_b.func_72872_a(EntityXPOrb.class, box);
        for (EntityXPOrb entity : xp) {
            double xDist = (func_174877_v().func_177958_n() + 0.5D - entity.field_70165_t);
            double yDist = (func_174877_v().func_177956_o() + 0.5D - entity.field_70163_u);
            double zDist = (func_174877_v().func_177952_p() + 0.5D - entity.field_70161_v);

            double totalDistance = Math.sqrt(xDist * xDist + yDist * yDist + zDist * zDist);

            if (totalDistance < 1.5) {
                hooverXP(entity);
            } else if (shouldAttract(func_174877_v(), entity)) {
                double d = 1 - (Math.max(0.1, totalDistance) / maxDist);
                double speed = 0.01 + (d * 0.02);

                entity.field_70159_w += xDist / totalDistance * speed;
                entity.field_70179_y += zDist / totalDistance * speed;
                entity.field_70181_x += yDist / totalDistance * speed;
                if (yDist > 0.5) {
                    entity.field_70181_x = 0.12;
                }

                // force client sync because this movement is server-side only
                boolean silent = entity.func_174814_R();
                entity.func_174810_b(!silent);
                entity.func_174810_b(silent);
            }
        }
    }

    private static final String PREVENT_REMOTE_MOVEMENT = "PreventRemoteMovement";
    public static final String BWM_PULLER_TAG = "BWMpuller";

    public static boolean shouldAttract(@Nullable BlockPos pullerPos, @Nullable Entity entity) {

        if (entity == null || entity.field_70128_L) {
            return false;
        }
        if (entity instanceof IProjectile && entity.field_70181_x > 0.01) {
            return false;
        }

        NBTTagCompound data = entity.getEntityData();

        if (isReservedByOthers(data)) {
            return false;
        }

        if (!isReservedByBWM(data)) {
            // if it is not being pulled already, pull it
            if (pullerPos != null) {
                data.func_74772_a(BWM_PULLER_TAG, pullerPos.func_177986_g());
            }
            return true;
        }

        if (pullerPos == null) {
            // it is already being pulled, so with no further info we are done
            return false;
        }

        long posL = data.func_74763_f(BWM_PULLER_TAG);
        if (posL == pullerPos.func_177986_g()) {
            // item already pulled from pullerPos so done
            return true;
        }

        // it is being pulled by something else, so check to see if we are closer
        BlockPos curOwner = BlockPos.func_177969_a(posL);
        double distToCur = curOwner.func_177957_d(entity.field_70165_t, entity.field_70163_u, entity.field_70161_v);
        double distToMe = pullerPos.func_177957_d(entity.field_70165_t, entity.field_70163_u, entity.field_70161_v);
        if (distToMe + 1 < distToCur) {
            // only take over if it is clearly nearer to us
            data.func_74772_a(BWM_PULLER_TAG, pullerPos.func_177986_g());
            return true;
        }
        return false;
    }

    public static void release(@Nullable Entity entity) {
        if (entity != null && !entity.field_70128_L) {
            NBTTagCompound data = entity.getEntityData();
            data.func_82580_o(BWM_PULLER_TAG);
        }
    }

    public static boolean isReserved(Entity entity) {
        return isReservedByBWM(entity.getEntityData()) || isReservedByOthers(entity.getEntityData());
    }

    public static boolean isReservedByBWM(NBTTagCompound data) {
        return data.func_74764_b(BWM_PULLER_TAG);
    }


    public static boolean isReservedByOthers(NBTTagCompound data) {
        return data.func_74764_b(PREVENT_REMOTE_MOVEMENT);
    }
}
