package betterwithmods.util;

import com.google.common.collect.Sets;
import net.minecraft.block.BlockLiquid;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.monster.EntityGhast;
import net.minecraft.init.Blocks;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.EnumSkyBlock;
import net.minecraft.world.World;
import net.minecraft.world.chunk.Chunk;
import net.minecraft.world.chunk.storage.ExtendedBlockStorage;

import java.util.Set;

import static net.minecraft.world.chunk.Chunk.NULL_BLOCK_STORAGE;

/**
 * @author Koward
 */
public final class WorldUtils {
    private WorldUtils() {
    }

    /**
     * Based on {@link World#getLightFromNeighbors(BlockPos)} build 2185
     */
    public static int getNaturalLightFromNeighbors(World worldIn, BlockPos pos) {
        return getNaturalLight(worldIn, pos, true, 0);
    }

    /**
     * Based on {@link World#getLight(BlockPos, boolean)} build 2185
     */
    private static int getNaturalLight(World worldIn, BlockPos pos, boolean checkNeighbors, int amount) {
        if (pos.func_177958_n() >= -30000000 && pos.func_177952_p() >= -30000000 && pos.func_177958_n() < 30000000 && pos.func_177952_p() < 30000000) {
            if (checkNeighbors && worldIn.func_180495_p(pos).func_185916_f()) {
                int i1 = getNaturalLight(worldIn, pos.func_177984_a(), false, 0);
                int i = getNaturalLight(worldIn, pos.func_177974_f(), false, 0);
                int j = getNaturalLight(worldIn, pos.func_177976_e(), false, 0);
                int k = getNaturalLight(worldIn, pos.func_177968_d(), false, 0);
                int l = getNaturalLight(worldIn, pos.func_177978_c(), false, 0);

                if (i > i1) {
                    i1 = i;
                }

                if (j > i1) {
                    i1 = j;
                }

                if (k > i1) {
                    i1 = k;
                }

                if (l > i1) {
                    i1 = l;
                }

                return i1;
            } else if (pos.func_177956_o() < 0) {
                return 0;
            } else {
                if (pos.func_177956_o() >= 256) {
                    pos = new BlockPos(pos.func_177958_n(), 255, pos.func_177952_p());
                }

                Chunk chunk = worldIn.func_175726_f(pos);
                return getNaturalLightSubtracted(chunk, pos, amount);
            }
        } else {
            return 15;
        }
    }

    /**
     * Based on {@link Chunk#getLightSubtracted(BlockPos, int)} build 2185
     */
    private static int getNaturalLightSubtracted(Chunk chunkIn, BlockPos pos, int amount) {
        int i = pos.func_177958_n() & 15;
        int j = pos.func_177956_o();
        int k = pos.func_177952_p() & 15;
        ExtendedBlockStorage extendedblockstorage = chunkIn.func_76587_i()[j >> 4];

        if (extendedblockstorage == field_186036_a) {
            return !chunkIn.func_177412_p().field_73011_w.func_177495_o() && amount < EnumSkyBlock.SKY.field_77198_c ? EnumSkyBlock.SKY.field_77198_c - amount : 0;
        } else {
            int l = chunkIn.func_177412_p().field_73011_w.func_177495_o() ? 0 : extendedblockstorage.func_76670_c(i, j & 15, k);
            l = l - amount;

            if (l < 0) {
                l = 0;
            }

            return l;
        }
    }

    public static int getDistance(BlockPos pos1, BlockPos pos2) {
        if (pos1 != null && pos2 != null)
            return (int) pos1.func_185332_f(pos2.func_177958_n(), pos2.func_177956_o(), pos2.func_177952_p());
        return 0;
    }

    public static boolean spawnGhast(World world, BlockPos pos) {
        EntityGhast ghast = new EntityGhast(world);

        for (int i = 0; i < 200; i++) {
            double xPos = pos.func_177958_n() + (world.field_73012_v.nextDouble() - world.field_73012_v.nextDouble()) * 30.0D;
            double yPos = pos.func_177956_o() + world.field_73012_v.nextInt(21) - 10;
            double zPos = pos.func_177952_p() + (world.field_73012_v.nextDouble() - world.field_73012_v.nextDouble()) * 30.0D;

            ghast.func_70012_b(xPos, yPos, zPos, world.field_73012_v.nextFloat() * 360.0F, 0.0F);

            if (ghast.func_70601_bi()) {
                world.func_72838_d(ghast);
                return true;
            }
        }
        return false;
    }

    public static boolean isWater(World world, BlockPos pos) {
        IBlockState state = world.func_180495_p(pos);
        return state.func_177230_c() == Blocks.field_150355_j || state.func_177230_c() == Blocks.field_150358_i;
    }

    public static boolean isWaterSource(World world, BlockPos pos) {
        IBlockState state = world.func_180495_p(pos);
        return state.func_177230_c() == Blocks.field_150355_j && state.func_177229_b(BlockLiquid.field_176367_b) == 0;
    }

    public static AxisAlignedBB toInts(AxisAlignedBB box) {
        return new AxisAlignedBB((int) box.field_72340_a, (int) box.field_72338_b, (int) box.field_72339_c, (int) box.field_72336_d, (int) box.field_72337_e, (int) box.field_72334_f);
    }

    public static Set<BlockPos> getPosAround(BlockPos pos, EnumFacing.Axis axis) {
        Set<BlockPos> posSet = Sets.newHashSet();
        for (int i = -1; i <= 1; i++) {
            for (int j = -1; j <= 1; j++) {
                switch (axis) {
                    case X:
                        posSet.add(pos.func_177982_a(0, i, j));
                        break;
                    case Y:
                        posSet.add(pos.func_177982_a(i, 0, j));
                        break;
                    case Z:
                        posSet.add(pos.func_177982_a(i, j, 0));
                        break;
                }
            }
        }
        return posSet;
    }

}
